unit uMainLog;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, uAppLog, ExtCtrls, ActnList, StdCtrls, ComCtrls, Buttons, IniFiles;

type
  TLogThread = class(TThread)
  private
    FSource : String;
  protected
    procedure Execute; override;
  public
  end;

  TfMainLog = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    Label1: TLabel;
    sbStart1: TSpeedButton;
    Label4: TLabel;
    sbStart2: TSpeedButton;
    lProcess1: TLabel;
    lProcess2: TLabel;
    pc: TPageControl;
    tsIniFile: TTabSheet;
    mIniFile: TMemo;
    Panel4: TPanel;
    sbProcessIniFile: TSpeedButton;
    ActionList1: TActionList;
    aStartStop1: TAction;
    aStartStop2: TAction;
    aProcessIni: TAction;
    tRefresh: TTimer;
    sbHelp: TSpeedButton;
    procedure aProcessIniExecute(Sender: TObject);
    procedure aStartStop1Execute(Sender: TObject);
    procedure aStartStop2Execute(Sender: TObject);
    procedure tRefreshTimer(Sender: TObject);
    procedure sbHelpClick(Sender: TObject);
  private
    { Private declarations }
    FAppDir : String;
    FTempFile : String;
    FLogs     : TStringList;
    FThread1  : TLogThread;
    FThread2  : TLogThread;
  protected
    function GenTempFileName : String;
    procedure InitIni;
    procedure RefreshLogs;
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  end;

var
  fMainLog: TfMainLog;

implementation

{$R *.dfm}

// TLogThread
procedure TLogThread.Execute;
  var b : Byte;
      i : Integer;
begin
  i := 0;
  repeat
    b := Random(Ord(High(TLogLevel))+1);
    Inc(i);
    AppLog(TlogLevel(b), FSource, 'Some event '+IntToStr(i));
    Sleep(1000);
  until Terminated;
end;

// Tfmain
constructor TfMainLog.Create(AOwner : TComponent);
begin
  inherited;
  FAppDir := ExtractFilePath(Application.ExeName);
  FAppDir := ExcludeTrailingPathDelimiter(FAppDir);
  FTempFile := GenTempFileName;
  InitIni;
  FLogs := TStringList.Create;
  FThread1 := nil;
  FThread2 := nil;
  aProcessIni.Execute;
end;

destructor TfMainLog.Destroy;
begin
  if Assigned(FThread2) then begin
    FThread2.Terminate;
    FreeAndNil(FThread2);
  end;
  if Assigned(FThread1) then begin
    FThread1.Terminate;
    FreeAndNil(FThread1);
  end;
  if Assigned(FLogs) then FreeAndNil(FLogs);
  inherited;
end;

function TfMainLog.GenTempFileName : String;
  var tmpFName : String;
      tmpPath  : String;
      PLen : Integer;
begin
  SetLength(tmpPath, MAX_PATH+1);
  PLen := GetTempPath(MAX_PATH, PChar(tmpPath));
  if PLen > 0 then SetLength(tmpPath, PLen) else tmpPath := FAppDir;
  SetLength(tmpFName, MAX_PATH+1);
  if GetTempFileName(PChar(tmpPath), 'aldemo', 0, PChar(tmpFName)) = 0
    then raise Exception.Create('Can''t create temp file!');
  SetLength(tmpFName, StrLen(PChar(tmpFName)));
  Result := tmpFName;
end;

procedure TfMainLog.InitIni;
  var iFile : TIniFile;
begin
  mIniFile.Lines.SaveToFile(FTempFile);
  iFile := TIniFile.Create(FTempFile);
  try
    iFile.WriteString('Global', 'Path', FAppDir);
  finally
    iFile.Free;
  end;
  mIniFile.Lines.LoadFromFile(FTempFile);
  DeleteFile(FTempFile);
end;

procedure TfMainLog.aProcessIniExecute(Sender: TObject);
  procedure CreateLogTabSheet(Path : String; LogFile : String);
    var i : Integer;
        ts : TTabSheet;
        m  : TMemo;
  begin
    i := FLogs.Add(LogFile);
    ts := TTabSheet.Create(pc);
    ts.PageControl := pc;
    ts.Caption := LogFile;
    m := TMemo.Create(ts);
    m.Parent := ts;
    m.Align := alClient;
    m.ScrollBars := ssVertical;
    m.Font := mIniFile.Font;
    m.Font.Color := clLime;
    m.Color := clBlack;
    m.Lines.Clear;
    m.Hint := IncludeTrailingPathDelimiter(Path)+LogFile;
    m.Lines.Add(m.Hint);
    m.ReadOnly := true;
    FLogs.Objects[i] := m;
  end;

  var i : Integer;
begin
  mIniFile.Lines.SaveToFile(FTempFile);
  try ProcessIniFile(FTempFile); except end;
  while pc.PageCount > 1 do pc.Pages[pc.PageCount-1].Free;
  FLogs.Clear;
  CreateLogTabSheet(LogDefs.Global.Path, LogDefs.Global.FileName);
  for i := 0 to LogDefs.Count-1 do
    CreateLogTabSheet(LogDefs.LogDef[i].Path, LogDefs.LogDef[i].FileName);
  DeleteFile(FTempFile);
  RefreshLogs;
end;

procedure TfMainLog.aStartStop1Execute(Sender: TObject);
begin
  if Assigned(FThread1) then begin
    FThread1.Terminate;
    FreeAndNil(FThread1);
    aStartStop1.Caption := 'Start';
    lProcess1.Visible := false;
  end
  else begin
    FThread1 := TLogThread.Create(true);
    FThread1.FSource := 'Thread1';
    aStartStop1.Caption := 'Stop';
    lProcess1.Visible := true;
    FThread1.Resume;
  end;
  aProcessIni.Enabled := not (Assigned(FThread1) or Assigned(FThread2));
  mIniFile.ReadOnly := not aProcessIni.Enabled;
  tRefresh.Enabled := not aProcessIni.Enabled;
  if tRefresh.Enabled and (pc.PageCount > 1) then pc.ActivePageIndex := 1;
end;

procedure TfMainLog.aStartStop2Execute(Sender: TObject);
begin
  if Assigned(FThread2) then begin
    FThread2.Terminate;
    FreeAndNil(FThread2);
    aStartStop2.Caption := 'Start';
    lProcess2.Visible := false;
  end
  else begin
    FThread2 := TLogThread.Create(true);
    FThread2.FSource := 'Thread2';
    aStartStop2.Caption := 'Stop';
    lProcess2.Visible := true;
    FThread2.Resume;
  end;
  aProcessIni.Enabled := not (Assigned(FThread1) or Assigned(FThread2));
  mIniFile.ReadOnly := not aProcessIni.Enabled;
  tRefresh.Enabled := not aProcessIni.Enabled;
  if tRefresh.Enabled and (pc.PageCount > 1) then pc.ActivePageIndex := 1;
end;

procedure TfMainLog.RefreshLogs;
  var i : Integer;
      m : TMemo;
begin
  for i := 0 to FLogs.Count - 1 do begin
    try
      m := TMemo(FLogs.Objects[i]);
      m.Lines.LoadFromFile(m.Hint);
      m.SelStart := Length(m.Text);
      m.Perform(EM_SCROLLCARET, 0, 0);
    except
    end;
  end;
end;

procedure TfMainLog.tRefreshTimer(Sender: TObject);
begin
  RefreshLogs;
end;

procedure TfMainLog.sbHelpClick(Sender: TObject);
begin
  WinHelp(Handle, pchar(FAppDir + '\eapplog.hlp'), HELP_CONTEXT, 205);
end;

end.
